import { Stack, Middleware } from './Stack';
import { Request, Response, RequestOptions } from './HTTPRequest';
export interface Context {
    request: Request;
    response: Response;
}
export interface ClientOptions extends RequestOptions {
    baseURI?: string;
}
export declare class HTTPClient<T extends Context> extends Stack<T> {
    baseURI: string;
    options: RequestOptions;
    constructor({ baseURI, ...options }?: ClientOptions);
    defaultWare: Middleware<T>;
    use(...middlewares: Middleware<T>[]): this;
    request<B>(data: T['request']): Promise<Response<B>>;
    head(path: Request['path'], headers?: Request['headers']): Promise<Record<string, string | object>>;
    get<B>(path: Request['path'], headers?: Request['headers']): Promise<Response<B>>;
    post<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers']): Promise<Response<B>>;
    put<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers']): Promise<Response<B>>;
    patch<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers']): Promise<Response<B>>;
    delete<B>(path: Request['path'], body?: Request['body'], headers?: Request['headers']): Promise<Response<B>>;
}
