import { Observable } from 'iterable-observer';
export declare enum BodyRequestMethods {
    POST = "POST",
    PUT = "PUT",
    PATCH = "PATCH",
    DELETE = "DELETE"
}
export interface RequestOptions {
    withCredentials?: boolean;
    timeout?: number;
    responseType?: XMLHttpRequestResponseType;
}
export interface Request extends RequestOptions {
    method?: 'HEAD' | 'GET' | keyof typeof BodyRequestMethods;
    path: string | URL;
    headers?: HeadersInit;
    body?: BodyInit | HTMLFormElement | any;
}
export interface Response<B = Request['body']> {
    status: number;
    statusText: string;
    headers: Record<string, string | object>;
    body?: B;
}
export declare class HTTPError<B = Request['body']> extends URIError {
    status: number;
    statusText: string;
    headers: Response<B>['headers'];
    body?: B;
    constructor(message: string, response: Response);
}
export type LinkHeader = Record<string, {
    URI: string;
    rel: string;
    title?: string;
}>;
export declare const headerParser: {
    Link(value: string): LinkHeader;
};
export declare function parseHeaders(raw: string): Response['headers'];
export interface RequestResult<B> {
    response: Promise<Response<B>>;
    upload?: Observable<ProgressEvent>;
    download?: Observable<ProgressEvent>;
}
export declare function requestXHR<B>({ method, path, headers, body, ...rest }: Request): RequestResult<B>;
export declare function requestFetch<B>({ path, method, headers, withCredentials, body, timeout, responseType }: Request): Promise<Response<B>>;
export declare function request<B>(options: Request): RequestResult<B>;
